@@ -3,10 +3,19 @@ from __future__ import annotations
 import typing as t
 
 from sqlglot import exp
-from sqlglot.dialects.dialect import rename_func, unit_to_var, timestampdiff_sql, build_date_delta
+from sqlglot.dialects.dialect import (
+    Version,
+    rename_func,
+    unit_to_var,
+    timestampdiff_sql,
+    build_date_delta,
+    date_delta_to_binary_interval_op,
+    groupconcat_sql,
+)
 from sqlglot.dialects.hive import _build_with_ignore_nulls
 from sqlglot.dialects.spark2 import Spark2, temporary_storage_provider, _build_as_cast
 from sqlglot.helper import ensure_list, seq_get
+from sqlglot.tokens import TokenType
 from sqlglot.transforms import (
     ctas_with_tmp_tables_to_create_tmp_view,
     remove_unique_constraints,
@@ -89,6 +98,17 @@ def _dateadd_sql(self: Spark.Generator, expression: exp.TsOrDsAdd | exp.Timestam
     return this
 
 
+def _groupconcat_sql(self: Spark.Generator, expression: exp.GroupConcat) -> str:
+    if self.dialect.version < Version("4.0.0"):
+        expr = exp.ArrayToString(
+            this=exp.ArrayAgg(this=expression.this),
+            expression=expression.args.get("separator") or exp.Literal.string(""),
+        )
+        return self.sql(expr)
+
+    return groupconcat_sql(self, expression)
+
+
 class Spark(Spark2):
     SUPPORTS_ORDER_BY_ALL = True
 
@@ -109,8 +129,12 @@ class Spark(Spark2):
             "DATEADD": _build_dateadd,
             "TIMESTAMPADD": _build_dateadd,
             "TIMESTAMPDIFF": build_date_delta(exp.TimestampDiff),
+            "TRY_ADD": exp.SafeAdd.from_arg_list,
+            "TRY_MULTIPLY": exp.SafeMultiply.from_arg_list,
+            "TRY_SUBTRACT": exp.SafeSubtract.from_arg_list,
             "DATEDIFF": _build_datediff,
             "DATE_DIFF": _build_datediff,
+            "LISTAGG": exp.GroupConcat.from_arg_list,
             "TIMESTAMP_LTZ": _build_as_cast("TIMESTAMP_LTZ"),
             "TIMESTAMP_NTZ": _build_as_cast("TIMESTAMP_NTZ"),
             "TRY_ELEMENT_AT": lambda args: exp.Bracket(
@@ -121,6 +145,16 @@ class Spark(Spark2):
             ),
         }
 
+        PLACEHOLDER_PARSERS = {
+            **Spark2.Parser.PLACEHOLDER_PARSERS,
+            TokenType.L_BRACE: lambda self: self._parse_query_parameter(),
+        }
+
+        def _parse_query_parameter(self) -> t.Optional[exp.Expression]:
+            this = self._parse_id_var()
+            self._match(TokenType.R_BRACE)
+            return self.expression(exp.Placeholder, this=this, widget=True)
+
         def _parse_generated_as_identity(
             self,
         ) -> (
@@ -139,6 +173,7 @@ class Spark(Spark2):
         SUPPORTS_CONVERT_TIMEZONE = True
         SUPPORTS_MEDIAN = True
         SUPPORTS_UNIX_SECONDS = True
+        SUPPORTS_DECODE_CASE = True
 
         TYPE_MAPPING = {
             **Spark2.Generator.TYPE_MAPPING,
@@ -163,11 +198,22 @@ class Spark(Spark2):
                     move_partitioned_by_to_schema_columns,
                 ]
             ),
+            exp.DateFromUnixDate: rename_func("DATE_FROM_UNIX_DATE"),
+            exp.DatetimeAdd: date_delta_to_binary_interval_op(cast=False),
+            exp.DatetimeSub: date_delta_to_binary_interval_op(cast=False),
+            exp.GroupConcat: _groupconcat_sql,
+            exp.EndsWith: rename_func("ENDSWITH"),
             exp.PartitionedByProperty: lambda self,
             e: f"PARTITIONED BY {self.wrap(self.expressions(sqls=[_normalize_partition(e) for e in e.this.expressions], skip_first=True))}",
+            exp.SafeAdd: rename_func("TRY_ADD"),
+            exp.SafeMultiply: rename_func("TRY_MULTIPLY"),
+            exp.SafeSubtract: rename_func("TRY_SUBTRACT"),
             exp.StartsWith: rename_func("STARTSWITH"),
+            exp.TimeAdd: date_delta_to_binary_interval_op(cast=False),
+            exp.TimeSub: date_delta_to_binary_interval_op(cast=False),
             exp.TsOrDsAdd: _dateadd_sql,
             exp.TimestampAdd: _dateadd_sql,
+            exp.TimestampSub: date_delta_to_binary_interval_op(cast=False),
             exp.DatetimeDiff: timestampdiff_sql,
             exp.TimestampDiff: timestampdiff_sql,
             exp.TryCast: lambda self, e: (
@@ -199,3 +245,9 @@ class Spark(Spark2):
                 return self.func("DATEDIFF", unit_to_var(expression), start, end)
 
             return self.func("DATEDIFF", end, start)
+
+        def placeholder_sql(self, expression: exp.Placeholder) -> str:
+            if not expression.args.get("widget"):
+                return super().placeholder_sql(expression)
+
+            return f"{{{expression.name}}}"
