@@ -1044,6 +1044,20 @@ class TestDuckDB(Validator):
                 "spark": "SELECT CONCAT(COALESCE(ARRAY('abc'), ARRAY()), ARRAY('bcg'))",
             },
         )
+        self.validate_identity(
+            "SELECT CUME_DIST( ORDER BY foo) OVER (ORDER BY 1) FROM (SELECT 1 AS foo)"
+        )
+        self.validate_identity(
+            "SELECT NTILE(1 ORDER BY foo) OVER (ORDER BY 1) FROM (SELECT 1 AS foo)"
+        )
+        self.validate_identity(
+            "SELECT RANK( ORDER BY foo) OVER (ORDER BY 1) FROM (SELECT 1 AS foo)"
+        )
+        self.validate_identity(
+            "SELECT PERCENT_RANK( ORDER BY foo) OVER (ORDER BY 1) FROM (SELECT 1 AS foo)"
+        )
+        self.validate_identity("LIST_COSINE_DISTANCE(x, y)")
+        self.validate_identity("LIST_DISTANCE(x, y)")
 
     def test_array_index(self):
         with self.assertLogs(helper_logger) as cm:
