@@ -885,6 +885,12 @@ GRANT SELECT, INSERT ON FUNCTION tbl TO user
 GRANT SELECT ON orders TO ROLE PUBLIC
 GRANT SELECT ON nation TO alice WITH GRANT OPTION
 GRANT DELETE ON SCHEMA finance TO bob
+REVOKE SELECT ON TABLE tbl FROM user
+REVOKE SELECT, INSERT ON FUNCTION tbl FROM user
+REVOKE SELECT ON orders FROM ROLE PUBLIC
+REVOKE GRANT OPTION FOR SELECT ON nation FROM alice
+REVOKE DELETE ON SCHEMA finance FROM bob CASCADE
+REVOKE INSERT ON TABLE orders FROM user RESTRICT
 SELECT attach
 SELECT detach
 SELECT 1 OFFSET 1
@@ -897,3 +903,28 @@ SELECT ARG_MAX(DISTINCT selected_col, filtered_col) FROM table
 SELECT ARG_MIN(DISTINCT selected_col, filtered_col) FROM table
 a.b.c.D()
 a.b.c.d.e.f.G()
+SELECT CUME_DIST() OVER (ORDER BY 1) FROM (SELECT 1)
+SELECT DENSE_RANK() OVER (ORDER BY 1) FROM (SELECT 1)
+SELECT NTILE(1) OVER (ORDER BY 1) FROM (SELECT 1)
+SELECT RANK() OVER (ORDER BY 1) FROM (SELECT 1)
+SELECT PERCENT_RANK() OVER (ORDER BY 1) FROM (SELECT 1)
+SELECT ACOS(x)
+SELECT ACOSH(x)
+SELECT ASIN(x)
+SELECT ASINH(x)
+SELECT ATAN(x)
+SELECT ATANH(x)
+SELECT ATAN2(x, y)
+SELECT COT(x)
+SELECT COTH(x)
+SELECT CSC(x)
+SELECT CSCH(x)
+SELECT SEC(x)
+SELECT SECH(x)
+SELECT UTC_DATE
+SELECT UTC_TIME
+SELECT SIN(x)
+SELECT SINH(x)
+SELECT COSINE_DISTANCE(v1, v2)
+SELECT EUCLIDEAN_DISTANCE(v1, v2)
+FOO(values.c)
